@@ -30,8 +30,8 @@ import warnings
 # smart_open.submodule to reference to the submodules.
 #
 import smart_open.local_file as so_file
+import smart_open.compression as so_compression
 
-from smart_open import compression
 from smart_open import doctools
 from smart_open import transport
 
@@ -106,7 +106,7 @@ def open(
         newline=None,
         closefd=True,
         opener=None,
-        ignore_ext=False,
+        compression=so_compression.INFER_FROM_EXTENSION,
         transport_params=None,
         ):
     r"""Open the URI object, returning a file-like object.
@@ -137,8 +137,8 @@ def open(
         Mimicks built-in open parameter of the same name.  Ignored.
     opener: object, optional
         Mimicks built-in open parameter of the same name.  Ignored.
-    ignore_ext: boolean, optional
-        Disable transparent compression/decompression based on the file extension.
+    compression: str, optional (see smart_open.compression.get_supported_compression_types)
+        Explicitly specify the compression/decompression behavior.
     transport_params: dict, optional
         Additional parameters for the transport layer (see notes below).
 
@@ -168,13 +168,16 @@ def open(
     if not isinstance(mode, str):
         raise TypeError('mode should be a string')
 
+    if compression not in so_compression.get_supported_compression_types():
+        raise ValueError(f'invalid compression type: {compression}')
+
     if transport_params is None:
         transport_params = {}
 
     fobj = _shortcut_open(
         uri,
         mode,
-        ignore_ext=ignore_ext,
+        compression=compression,
         buffering=buffering,
         encoding=encoding,
         errors=errors,
@@ -219,10 +222,7 @@ def open(
         raise NotImplementedError(ve.args[0])
 
     binary = _open_binary_stream(uri, binary_mode, transport_params)
-    if ignore_ext:
-        decompressed = binary
-    else:
-        decompressed = compression.compression_wrapper(binary, binary_mode)
+    decompressed = so_compression.compression_wrapper(binary, binary_mode, compression)
 
     if 'b' not in mode or explicit_encoding is not None:
         decoded = _encoding_wrapper(
@@ -235,6 +235,19 @@ def open(
     else:
         decoded = decompressed
 
+    #
+    # There are some useful methods in the binary readers, e.g. to_boto3, that get
+    # hidden by the multiple layers of wrapping we just performed.  Promote
+    # them so they are visible to the user.
+    #
+    if decoded != binary:
+        promoted_attrs = ['to_boto3']
+        for attr in promoted_attrs:
+            try:
+                setattr(decoded, attr, getattr(binary, attr))
+            except AttributeError:
+                pass
+
     return decoded
 
 
@@ -295,7 +308,7 @@ def _get_binary_mode(mode_str):
 def _shortcut_open(
         uri,
         mode,
-        ignore_ext=False,
+        compression,
         buffering=-1,
         encoding=None,
         errors=None,
@@ -309,12 +322,13 @@ def _shortcut_open(
     This is only possible under the following conditions:
 
         1. Opening a local file; and
-        2. Ignore extension is set to True
+        2. Compression is disabled
 
     If it is not possible to use the built-in open for the specified URI, returns None.
 
     :param str uri: A string indicating what to open.
     :param str mode: The mode to pass to the open function.
+    :param str compression: The compression type selected.
     :returns: The opened file
     :rtype: file
     """
@@ -326,8 +340,11 @@ def _shortcut_open(
         return None
 
     local_path = so_file.extract_local_path(uri)
-    _, extension = P.splitext(local_path)
-    if extension in compression.get_supported_extensions() and not ignore_ext:
+    if compression == so_compression.INFER_FROM_EXTENSION:
+        _, extension = P.splitext(local_path)
+        if extension in so_compression.get_supported_extensions():
+            return None
+    elif compression != so_compression.NO_COMPRESSION:
         return None
 
     open_kwargs = {}
@@ -364,16 +381,16 @@ def _open_binary_stream(uri, mode, transport_params):
         #
         raise NotImplementedError('unsupported mode: %r' % mode)
 
-    if hasattr(uri, 'read'):
-        # simply pass-through if already a file-like
-        # we need to return something as the file name, but we don't know what
-        # so we probe for uri.name (e.g., this works with open() or tempfile.NamedTemporaryFile)
-        # if the value ends with COMPRESSED_EXT, we will note it in compression_wrapper()
-        # if there is no such an attribute, we return "unknown" - this
-        # effectively disables any compression
-        if not hasattr(uri, 'name'):
-            uri.name = getattr(uri, 'name', 'unknown')
-        return uri
+    if isinstance(uri, int):
+        #
+        # We're working with a file descriptor.  If we open it, its name is
+        # just the integer value, which isn't helpful.  Unfortunately, there's
+        # no easy cross-platform way to go from a file descriptor to the filename,
+        # so we just give up here.  The user will have to handle their own
+        # compression, etc. explicitly.
+        #
+        fobj = _builtin_open(uri, mode, closefd=False)
+        return fobj
 
     if not isinstance(uri, str):
         raise TypeError("don't know how to handle uri %s" % repr(uri))
@@ -466,7 +483,7 @@ def smart_open(
     # For completeness, the main differences of the old smart_open function:
     #
     # 1. Default mode was read binary (mode='rb')
-    # 2. ignore_ext parameter was called ignore_extension
+    # 2. compression parameter was called ignore_extension
     # 3. Transport parameters were passed directly as kwargs
     #
     url = 'https://github.com/RaRe-Technologies/smart_open/blob/develop/MIGRATING_FROM_OLDER_VERSIONS.rst'
@@ -478,7 +495,10 @@ def smart_open(
     message = 'This function is deprecated.  See %s for more information' % url
     warnings.warn(message, category=DeprecationWarning)
 
-    ignore_ext = ignore_extension
+    if ignore_extension:
+        compression = so_compression.NO_COMPRESSION
+    else:
+        compression = so_compression.INFER_FROM_EXTENSION
     del kwargs, url, message, ignore_extension
     return open(**locals())
 
